const fs = require('fs')
const path = require('path')
const report = require('gatsby-cli/lib/reporter')
const firebase = require('firebase-admin')
const crypto = require('crypto')

const getDigest = id =>
  crypto
    .createHash('md5')
    .update(id)
    .digest('hex')

const writeDataFile = mappedData => {
  // console.log(`${type}: ${JSON.stringify(mappedData, null, 2)}`)
  if (mappedData.path) {
    const dir = path.dirname(mappedData.path)
    const writePath = path.join(process.cwd(), `./${mappedData.path}`)
    const dirPath = path.join(process.cwd(), `./${dir}`)
    if (!fs.existsSync(dirPath)) fs.mkdirSync(dirPath, { recursive: true })
    fs.writeFileSync(writePath, mappedData.raw, { flag: 'w', encoding: 'utf8' })
  }
}

exports.sourceNodes = async ({ boundActionCreators, reporter, actions }, { types, typeDefs, credential }) => {
  reporter.info(`Adding data nodes.`)
  const { createTypes } = actions
  /**
   * If type includes typeDefs, use createTypes to create schema
   * types to inform Gatsby of the data's shape.
   * See: https://www.gatsbyjs.org/blog/2019-03-18-releasing-new-schema-customization/
   */
  if (typeDefs) {
    reporter.info(`Defining typeDefs`)
    createTypes(typeDefs)
  }

  try {
    if (firebase.apps || !firebase.apps.length) {
      firebase.initializeApp({
        credential: firebase.credential.cert(credential),
      })
    }
  } catch (e) {
    report.warn(
      'Could not initialize Firebase. Please check `credential` property in gatsby-config.js',
    )
    report.warn(e)
    return
  }
  const db = firebase.firestore()
  db.settings({
    timestampsInSnapshots: true,
  })

  const { createNode } = boundActionCreators

  const promises = types.map(
    async ({ collection, type, map = node => node, files = false, write = false }) => {
      const snapshot = await db.collection(collection).get()
      if (files) {
        reporter.info(`Defining node ${type}`)
        // File Collections (unique typeDefs)
        let mappedData = { }
        const id = `${type}_file_collection`
        const contentDigest = getDigest(id)
        for (let doc of snapshot.docs) {
          reporter.info(`Defining ${type}.${doc.id}`)
          const fileData = map(doc.data())
          // if write===true we will write the data down to the folder path.
          if (write) writeDataFile(fileData)
          mappedData[doc.id] = fileData
        }
        createNode(
          Object.assign({}, mappedData, {
            id,
            parent: null,
            children: [],
            internal: {
              type,
              contentDigest,
            },
          }),
        )
      } else {
        // Collections Folder (same typeDefs)
        for (let doc of snapshot.docs) {
          reporter.info(`Defining node ${doc.id}`)
          const contentDigest = getDigest(doc.id)
          const mappedData = map(doc.data())
          createNode(
            Object.assign({}, mappedData, {
              id: doc.id,
              parent: null,
              children: [],
              internal: {
                type,
                contentDigest,
              },
            }),
          )
          // if write===true we will write the data down to the folder path.
          if (write) writeDataFile(mappedData)
        }
      }
      Promise.resolve()
    }
  )
  await Promise.all(promises)
  return
}
